//+-------------------------------------------------------------------------------------+
//|                                                                    Parabolic_V3.mq4 |
//|                                                           Scriptong_edit_RomanBesT  |
//|                                                                                     |
//+-------------------------------------------------------------------------------------+
#property copyright "Scriptong_edit_RomanBesT, 2010"
#property link      "http://fortrader.ru"

//---- input parameters
extern double       ParabolicStep=0.006;                       //   
extern double       ParabolicMax=0.05;                    //   
extern int          ATRPeriod = 24;                                         //  ATR

extern double       Lots = 0.1;
extern double       Mul = 5;                      //  ATR   

extern string       OpenOrderSound = "ok.wav";
extern int          MagicNumber = 10001;

bool Activate, FreeMarginAlert = False, FatalError = False;
double MinLot, LotStep, MaxLot, Tick, StopLevel, Spread, SL, TP;
int LastTicket, Signal;
datetime LastBar = 0;
static int prevtime = 0;     // For Price - Open

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   Activate = False;
// - 1 - ==      ========================================   
   Tick = MarketInfo(Symbol(), MODE_TICKSIZE);                         //      
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                 //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //    
   MinLot = MarketInfo(Symbol(), MODE_MINLOT);    //    
   MaxLot = MarketInfo(Symbol(), MODE_MAXLOT);   //    
   LotStep = MarketInfo(Symbol(), MODE_LOTSTEP);          //    
// - 1 - ==   =============================================================

// - 2 - ==          =======   
   Lots = MathRound(Lots/LotStep)*LotStep; //     
   if(Lots < MinLot || Lots > MaxLot) //     MinLot    MaxLot
     {
      Comment(" Lots     !  !");
      return(0);
     }
// - 2 - ==   =============================================================

// - 3 - =======================     =====================
   LastTicket = -1;
   for (int i = 0; i < OrdersTotal(); i++)
     if (OrderSelect(i, SELECT_BY_POS))
       if (Symbol() == OrderSymbol() && OrderMagicNumber() == MagicNumber)
         {
          LastTicket = OrderTicket();
          break;
         }
// - 3 - ==   =============================================================

   Activate = True;   
 
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+-------------------------------------------------------------------------------------+
//|                                                         |
//+-------------------------------------------------------------------------------------+
string ErrorToString(int Error)
{
 switch(Error)
   {
    case 2: return("  ,   ."); 
    case 5: return("    ,  ."); 
    case 6: return("   ,   ."); 
    case 64: return(" ,   .");
    case 132: return(" ."); 
    case 133: return(" ."); 
    case 149: return(" ."); 
   }
}

//+-------------------------------------------------------------------------------------+
//|   .   ,   True,  - False    |
//+-------------------------------------------------------------------------------------+  
bool WaitForTradeContext()
{
 int P = 0;
 //  ""
 while(IsTradeContextBusy() && P < 5)
   {
    P++;
    Sleep(1000);
   }
 // -------------  
 if(P == 5)
   return(False);
 return(True);    
}
  
//+-------------------------------------------------------------------------------------+
//| ""                                                         |
//|    OpenOrder            |
//| :                                                                         |
//|   0 -                                                                      |
//|   1 -                                                                 |
//|   2 -   Price                                                         |
//|   3 -   SL                                                            |
//|   4 -   TP                                                            |
//|   5 -   Lot                                                           |
//+-------------------------------------------------------------------------------------+
int OpenOrderCorrect(int Type, double Price, double SL, double TP,
                     bool Redefinition = True)
// Redefinition -  True     
//                 False -  
{
// - 1 - ==     ====================================
 if(AccountFreeMarginCheck(Symbol(), OP_BUY, Lots) <= 0 || GetLastError() == 134) 
  {
   if(!FreeMarginAlert)
    {
     Print("    . Free Margin = ", 
           AccountFreeMargin());
     FreeMarginAlert = True;
    } 
   return(5);  
  }
 FreeMarginAlert = False;  
// - 1 - ==   =============================================================

// - 2 - ==   Price, SL  TP    ====================   

 RefreshRates();
 switch (Type)
   {
    case OP_BUY: 
                string S = "BUY"; 
                if (MathAbs(Price-Ask)/Point > 3)
                  if (Redefinition) Price = ND(Ask);
                  else              return(2);
                if (ND(TP-Bid) <= StopLevel && TP != 0)
                  if (Redefinition) TP = ND(Bid+StopLevel+Tick);
                  else              return(4);
                if (ND(Bid-SL) <= StopLevel)
                  if (Redefinition) SL = ND(Bid-StopLevel-Tick);
                  else              return(3);
                break;
    case OP_SELL: 
                 S = "SELL"; 
                 if (MathAbs(Price-Bid)/Point > 3)
                   if (Redefinition) Price = ND(Bid);
                   else              return(2);
                 if (ND(Ask-TP) <= StopLevel) 
                   if (Redefinition) TP = ND(Ask-StopLevel-Tick);
                   else              return(4);
                 if (ND(SL-Ask) <= StopLevel && SL != 0)
                   if (Redefinition) SL = ND(Ask+StopLevel+Tick);
                   else              return(3);
                 break;
    case OP_BUYSTOP: 
                    S = "BUYSTOP";
                    if (ND(Price-Ask) <= StopLevel)
                      if (Redefinition) Price = ND(Ask+StopLevel+Tick);
                      else              return(2);
                    if (ND(TP-Price) <= StopLevel && TP != 0)
                      if (Redefinition) TP = ND(Price+StopLevel+Tick);
                      else              return(4);
                    if (ND(Price-SL) <= StopLevel)
                      if (Redefinition) SL = ND(Price-StopLevel-Tick);
                      else              return(3);
                    break;
    case OP_SELLSTOP: 
                     S = "SELLSTOP";
                     if (ND(Bid-Price) <= StopLevel)
                       if (Redefinition) Price = ND(Bid-StopLevel-Tick);
                       else              return(2);
                     if (ND(Price-TP) <= StopLevel)
                       if (Redefinition) TP = ND(Price-StopLevel-Tick);
                       else              return(4);
                     if (ND(SL-Price) <= StopLevel && SL != 0)
                       if (Redefinition) SL = ND(Price+StopLevel+Tick);
                       else              return(3);
                     break;
    case OP_BUYLIMIT: 
                     S = "BUYLIMIT";
                     if (ND(Ask-Price) <= StopLevel)
                      if (Redefinition) Price = ND(Ask-StopLevel-Tick);
                      else              return(2);
                     if (ND(TP-Price) <= StopLevel && TP != 0)
                       if (Redefinition) TP = ND(Price+StopLevel+Tick);
                       else              return(4);
                     if (ND(Price-SL) <= StopLevel)
                       if (Redefinition) SL = ND(Price-StopLevel-Tick);
                       else              return(3);
                     break;
    case OP_SELLLIMIT: 
                     S = "SELLLIMIT";
                     if (ND(Price - Bid) <= StopLevel) 
                       if (Redefinition) Price = ND(Bid+StopLevel+Tick);
                       else              return(2);
                     if (ND(Price-TP) <= StopLevel)
                       if (Redefinition) TP = ND(Price-StopLevel-Tick);
                       else              return(4);
                     if (ND(SL-Price) <= StopLevel && SL != 0)
                       if (Redefinition) SL = ND(Price+StopLevel+Tick);
                       else              return(3);
                     break;
   }
// - 2 - ==   =============================================================
 
// - 3 - ==       =================================   
 if(WaitForTradeContext())  //    
   {  
    Comment("     ", S, " ...");  
    int ticket=OrderSend(Symbol(), Type, Lots, Price, 30, 
               SL, TP, "Parabolic_V3", MagicNumber, 0);//  
    //     
    if(ticket<0)
      {
       int Error = GetLastError();
       if(Error == 2 || Error == 5 || Error == 6 || Error == 64 
          || Error == 132 || Error == 133 || Error == 149)     //   
         {
          Comment("     . . "+
                   ErrorToString(Error)+"  !");
          FatalError = True;
         }
        else 
         Comment("   ", S, ": ", Error);       //  
       return(1);
      }
    // ---------------------------------------------
    
    //      
    Comment(" ", S, "  !"," MagicNumber", MagicNumber); 
    PlaySound(OpenOrderSound);
    if (OrderSelect(ticket, SELECT_BY_TICKET) && OrderCloseTime() == 0)
      LastTicket = ticket; 
    return(0); 
    // ------------------------
   }
  else
   {
    Comment("     !");
    return(1);  
   } 
// - 3 - ==   =============================================================  
}

//+-------------------------------------------------------------------------------------+
//|     ,                                    |
//| :                                                                         |
//|   0 -    (  )                               |
//|   1 -                                                      |
//|   2 -                                                      |
//+-------------------------------------------------------------------------------------+
int CloseIfExists(int Type)
{
 if (OrderSelect(LastTicket, SELECT_BY_TICKET) && OrderCloseTime() == 0 && OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber )
    if (OrderType() == Type)
      {
       if(Type == OP_BUY)
         double Price = Bid;
        else
         Price = Ask; 
       if (WaitForTradeContext())  
         if (!OrderClose(OrderTicket(), OrderLots(), ND(Price), 10))
           return(2);
      }
     else 
      return(1); 

 return(0);       
}

//+-------------------------------------------------------------------------------------+
//|                                                |
//+-------------------------------------------------------------------------------------+
double ND(double A)
{
 return(MathRound(A/Tick)*Tick);
}  
  
//+-------------------------------------------------------------------------------------+
//|                                                                        |
//+-------------------------------------------------------------------------------------+  
void GetSignal() 
{
 Signal = 0;                                             //    

// - 1 -  ==   Parabolic SAR ==============================================
 double SAR1 = iSAR(Symbol(), 0, ParabolicStep, ParabolicMax, 1);
 double SAR2 = iSAR(Symbol(), 0, ParabolicStep, ParabolicMax, 2);
// - 1 -  ==   ============================================================

// - 2 -  ========================   SELL   BUY =================
 if (SAR1 < Open[1])
   {
    Signal = 2;                                                   //   SELL
    SL = ND(SAR1 - Tick);                                         //    BUY
    if (SAR2 > Open[2])                                            //   BUY
      Signal = 1;
   } 
// - 2 -  ==   ============================================================

// - 3 -  ========================   BUY   SELL =================
 if (SAR1 > Open[1])
   {
    Signal = -2;                                                   //   BUY
    SL = ND(SAR1 + Spread*Tick + Tick);                          //    SELL
    if ( SAR2 < Open[2])                                          //   SELL
       Signal = -1;
   } 
// - 3 -  ==   ============================================================
}
  
//+-------------------------------------------------------------------------------------+
//| expert start function                                                               |
//+-------------------------------------------------------------------------------------+
int start()
  {
// - 1 -  ==    ? ===========================================
   if (!Activate || FatalError)             //   ,  
    return(0);           //  init         
// - 1 -  ==   ============================================================
 if(Time[0] == prevtime)   return(0); //  
   prevtime = Time[0];     

// - 2 - ==      ========================================
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                  //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //    
// - 2 -  ==   ============================================================

// - 3 - ===============          ======
   if(LastBar == Time[0])
     return(0);
   GetSignal();  
// - 3 -  ====================================   ==========================
     
// - 4 - ===============  BUY   SELL =================================
   if (Signal > 0)
     {
      int Res = CloseIfExists(OP_SELL);
      if (Res == 2) return(0);
      if (Signal == 1 && Res == 0)
        {
         RefreshRates();
         double TP = ND(Ask + Mul*iATR(Symbol(), 0, ATRPeriod, 1));
         if (OpenOrderCorrect(OP_BUY, ND(Ask), SL, TP) != 0)
           return(0);
        }     
     }   
// - 4 -  ======================   ========================================

// - 5 - ===============  SELL   BUY =================================
   if (Signal < 0)
     {
      Res = CloseIfExists(OP_BUY);
      if (Res == 2) return(0);
      if (Signal == -1 && Res == 0)
        {
         RefreshRates();
         TP = ND(Bid - Mul*iATR(Symbol(), 0, ATRPeriod, 1));
         if (OpenOrderCorrect(OP_SELL, ND(Bid), SL, TP) != 0)
           return(0);
        } 
     }
// - 5 -  ======================   ========================================

// - 6 - =======================   ========================================
  if (OrderSelect(LastTicket, SELECT_BY_TICKET) && OrderCloseTime() == 0 && OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)
    {
     if (OrderType() == OP_BUY && SL > ND(OrderStopLoss()))
       if (WaitForTradeContext())
         if (ND(Bid - SL - StopLevel) > 0)
           if (!OrderModify(OrderTicket(), 0, SL, OrderTakeProfit(), 0))
             return(0);   
       
     if (OrderType() == OP_SELL && SL < ND(OrderStopLoss()))
       if (ND(SL - Ask - StopLevel) > 0)
         if (!OrderModify(OrderTicket(), 0, SL, OrderTakeProfit(), 0))
           return(0);
    }
// - 6 -  ======================   ========================================

  LastBar = Time[0]; 
//----
   return(0);
  }
//+------------------------------------------------------------------+